@@ -63,10 +63,10 @@ export class VideoPlayer extends Component {
     this._onChange = this._onChange.bind(this);
   }
 
-  getPreviewForSecond(forSecond = 0, maximumSize) {
+  getPreviewForSecond(forSecond = 0, maximumSize, format = 'base64') {
     const actualSource = getActualSource(this.props.source);
     return new Promise((resolve, reject) => {
-      RNVideoTrimmer.getPreviewImageAtPosition(actualSource, forSecond, maximumSize,
+      RNVideoTrimmer.getPreviewImageAtPosition(actualSource, forSecond, maximumSize, format,
         (err, base64) => {
           if (err) {
             return reject(err);
